## v4.0.0 Migration Guide

### Drop bower support

Since v4.0.0, tui.tree does not support [bower](https://bower.io/) anymore. You can use [npm](https://www.npmjs.com/) and CDN instead of Bower. To know how to install tui.tree by npm or CDN, please refer to the [💾 Install section](https://github.com/nhn/tui.tree#-install).

### Remove jQuery

In v4.0.0, jQuery is removed. It might cause some errors if you pass a jQuery object as a `container`.

Previously, you can use the jQuery object (`$`) to create an instance.

```javascript
// v3
const tree = new Tree($('#tree'), options);
```

Now, you have to use `selector` or `HTMLElement` as a `container`.

```javascript
// v4
const tree = new Tree('#tree', options);

// or

const container = document.getElementById('tree');
const tree = new Tree(container, options);
```

### Use the selector, not id selector

When a string is passed into a `container`, it was considered as the **id of the container** in v3. Now, it is considered as the **selector**.

```javascript
// v3
const tree = new Tree('tree', options);

// v4
const tree = new Tree('#tree', options);
```

### Change Ajax feature options

By replacing the Ajax module (`$.ajax()` -> [tui-code-snippet/ajax](https://nhn.github.io/tui.code-snippet/2.3.0/ajax)), some options have been renamed or removed.

* `command[type].type` -> `command[type].method`: The method of the Ajax request
* `command[type].dataType` -> `command[type].contentType`: The media type of the resource ([MDN reference](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Type))
* `command[type].jsonpCallback` -> removed
* `command[type].data` -> `command[type].params`: The parameters to be sent by the Ajax request

For more details, please refer to the [ajax-feature](./ajax-feature.md).

### Rename event properties

Some event properties are renamed in v4.0:

| Event | v3 | v4 |
| --- | --- | --- |
| [beforeAjaxRequest](https://nhn.github.io/tui.tree/4.0.0/Tree#event-beforeAjaxRequest) | type | command |
| | params | data |
| [successAjaxResponse](https://nhn.github.io/tui.tree/4.0.0/Tree#event-successAjaxResponse) | type | command |
| [failAjaxResponse](https://nhn.github.io/tui.tree/4.0.0/Tree#event-failAjaxResponse) | type | command |
| [errorAjaxResponse](https://nhn.github.io/tui.tree/4.0.0/Tree#event-errorAjaxResponse) | type | command |
| [selectContextMenu](https://nhn.github.io/tui.tree/4.0.0/Tree#event-selectContextMenu) | cmd | command |
